use std::fs;
use serde_json::Value;
use color_eyre::eyre::Result;

#[cfg(feature = "avatar")]
use {
    std::{
        process::Command,
        fs::remove_file,
        path::Path,
    },
    crate::{
        configdata::Config,
        app::App,
        clap_ps::ForceFrom,
    },
    color_eyre::eyre::eyre,
};

#[cfg(feature = "fedi")]
use crate::api::fedi::*;

pub fn load_json(file_path: &str) -> Result<Value> {

    let file_data = fs::read_to_string(&file_path)?;
    let json_value: Value = serde_json::from_str(&file_data)?;

    Ok(json_value)
}

#[cfg(feature = "avatar")]
pub fn update_avatars(app: &mut App, #[cfg(feature = "discord")] discord: bool, #[cfg(feature = "fedi")] fedi: bool) -> Result<()>{

    if let Some(mut cfg) = app.cfg.clone() {
        let names = &app.get_front(ForceFrom::None)?; // TODO CHANGE INTO USING LOCAL FRONT

        match avatar_module(&cfg, &names) {
            Ok(_) => {
                #[cfg(feature = "discord")] {
                    cfg.disc_module.enabled |= discord;
                    discord_module(&cfg)?;
                }
                #[cfg(feature = "fedi")] {
                    cfg.fedi_module.enabled |= fedi;
                    fedi_module(&cfg)?;
                }
            },
            Err(e) => println!("{}", e),
        }
        return Ok(());
    }
    Err(eyre!("No config found"))
}

#[cfg(feature = "avatar")]
fn avatar_module(config: &Config, names: &Vec<String>) -> Result<()>{
    if config.avatar_module.enabled {
        let mut whitelisted_names: Vec<String> = names.iter().map(|i| i.to_lowercase()).collect();
        let blacklist: Vec<String> = config.avatar_module.blacklist.iter().map(|i| i.to_lowercase()).collect();
        for name in names {
            if blacklist.contains(&name.to_lowercase()) {
                let index = whitelisted_names.iter().position(|x| *x == name.to_lowercase()).unwrap();
                whitelisted_names.remove(index);
                println!("{} blacklisted from avatar module", name);
            }
        }

        let avatarnames = whitelisted_names.join("").to_lowercase() + ".png";

        if Path::new(&config.avatar_module.avatar_output_path).exists() {
            let _ = remove_file(&config.avatar_module.avatar_output_path);
        }

        if cfg!(target_os = "windows") {
            let cmdaug = format!("copy {}\\{} {}", &config.avatar_module.avatar_folder, avatarnames, &config.avatar_module.avatar_output_path);
            Command::new("cmd").args(["/C", &cmdaug]).output().expect("Avatar module error");
        } else {
            Command::new("sh").arg("-c").arg(format!("cp {}/{} {}", &config.avatar_module.avatar_folder, avatarnames, &config.avatar_module.avatar_output_path)).output().expect("Avatar module error");
        }
        if Path::new(&config.avatar_module.avatar_output_path).exists() {
            println!("Avatar module finished");
            Ok(())
        } else {
            Err(eyre!("Avatar module failed"))
        }
    } else {
        Err(eyre!("Avatar mode disabled"))
    }

}

#[cfg(feature = "discord")]
fn discord_module(config: &Config) -> Result<()> {
    if config.disc_module.enabled {
        if cfg!(target_os = "windows") {
            let mut c = Command::new("cmd").args(["/C", format!("{} {}", &config.disc_module.python_path, &config.disc_module.script_path).as_str()]).spawn()?;
            let _ = c.wait()?;
        } else {
            let mut c = Command::new("sh").arg("-c").arg(format!("{} {}", &config.disc_module.python_path, &config.disc_module.script_path)).spawn()?;
            let _ = c.wait()?;
        }
    }
    Ok(())
}

#[cfg(feature = "fedi")]
fn fedi_module(config: &Config) -> Result<()> {
    if config.fedi_module.enabled {
        fedi_request(config)?;
    }

    Ok(())
}
